/**
 *    This file is part of OtviAI.
 *
 *    OtviAI is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    OtviAI is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with OtviAI.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright 2008-2010 Otto Visser.
 * Suggestions/comments and bugs can go to: otviAI@otvi.nl
 */

/**
 * list of tested NEWGrf's:
 * - ECS
 * - HEQS (combination with ECS is borked)
 */

/**
 * TODO:
 - do the whole thing in a transaction to try if you can do it for the money you have now?
 - check roads to depots; always generates err_unknown: aka I'm ignoring those also when I shouldn't
 - search with increasing radius: subtract inner circle to prevent that to be checked 10 times
 - max route cost based on max height found on map
 - add service to order?
 - support replacing vehicles if autorenew isn't working
 - check: depot heigth == track height
 - improve train depot building
 - reuse of town train stations
 - seperate depot building from track building. Perhaps list of stations associated with depots again.
 - remove stations no longer in use
 - add boat support
 - BUG: building passenger train for oil well???
 - combination HEQS and ECS seems borked
 */

require("util.nut");
require("town.nut");
require("RoadFinder/RoadFinder.nut");
require("RailFinder/RailFinder.nut");

class OtviAI extends AIController {
	biggestTown = -1;	
	subsidyBuilt = false;	// TODO fix: value is incorrected in some cases
	
	myTownList = null;
	myStationsTileList = null;
	myLorryTileList = null;
	myDepotsTileList = null;
	
	passengerCargoId = -1;
	mailCargoId = -1;
	
	passengerBusEngine = -1;
	mailLorryEngine = -1;
	trainEngine = -1;
	passengerCart = -1;
	mailCart = -1;
	
	lastMail1 = -1;
	lastMail2 = -1;
	lastMailVehicle = -1;
	lastTown1 = -1;
	lastTown2 = -1;
	lastPassengerVehicle = -1;
	lastMaxBusses = 0;
	expandMoneyMin = -1;
	
	roadFinder = null;
	railFinder = null;

	MIN_DISTANCE = 50;	// ends at 40
	MAX_DISTANCE = 100;	// ends at 200

	current_rail_type = 0;
	town = null;

	uberstartTick = 0;

	function Save() {
		// TODO
		return {};
	}

	function Load(version, data) {
		Debug("TODO: implement load and save");
	}

	function constructor() {
		roadFinder = RoadFinder();
		railFinder = RailMaker();
		town = OtviTown();

		myStationsTileList = AIList();
		myLorryTileList = AIList();
		myDepotsTileList = AIList();
		myTownList = AITownList();

		roadFinder.cost.tile = 1;
		roadFinder.cost.max_cost = 30000;		// TODO: still an estimate
		roadFinder.cost.no_existing_road = 5;	// The cost that is added to _cost_tile if no road exists yet.
		roadFinder.cost.turn = 10;				// The cost that is added to _cost_tile if the direction changes.
		roadFinder.cost.slope =	15;				// The extra cost if a road tile is sloped.
		roadFinder.cost.bridge_per_tile = 20;	// The cost per tile of a new bridge, this is added to _cost_tile.
		roadFinder.cost.tunnel_per_tile = 20;	// The cost per tile of a new tunnel, this is added to _cost_tile.
		roadFinder.cost.coast =	20;				// The extra cost for a coast tile.
		roadFinder.cost.max_bridge_length = 10;	// The maximum length of a bridge that will be build.
		roadFinder.cost.max_tunnel_length = 10;	// The maximum length of a tunnel that will be build.
		
		railFinder.cost.tile = 1;
		railFinder.cost.max_cost = 30000;	// TODO: still an estimate
		railFinder.cost.turn = 10;		// The cost that is added to _cost_tile if the direction changes.
		railFinder.cost.diagonal_tile = 0;
		railFinder.cost.new_rail = 5;
		railFinder.cost.road_tile = 5;
		railFinder.cost.slope =	10;		// The extra cost if a rail tile is sloped.
		railFinder.cost.bridge_per_tile = 20;	// The cost per tile of a new bridge, this is added to _cost_tile.
		railFinder.cost.tunnel_per_tile = 20;	// The cost per tile of a new tunnel, this is added to _cost_tile.
		railFinder.cost.coast =	15;		// The extra cost for a coast tile.
		railFinder.cost.max_bridge_length = 10;	// The maximum length of a bridge that will be build.
		railFinder.cost.max_tunnel_length = 10;	// The maximum length of a tunnel that will be build.

		local list = AICargoList();
		for (local i = list.Begin(); list.HasNext(); i = list.Next()) {
			if (AICargo.HasCargoClass(i, AICargo.CC_PASSENGERS) && AICargo.GetTownEffect(i) == AICargo.TE_PASSENGERS) {
				Debug("Found passenger cargo: " + AICargo.GetCargoLabel(i));
				passengerCargoId = i;
			}
		}

		list = AICargoList();
		for (local i = list.Begin(); list.HasNext(); i = list.Next()) {
			if (AICargo.HasCargoClass(i, AICargo.CC_MAIL) && AICargo.GetTownEffect(i) == AICargo.TE_MAIL) {
				Debug("Found mail cargo: " + AICargo.GetCargoLabel(i));
				mailCargoId = i;
			}
		}

		AIRoad.SetCurrentRoadType(AIRoad.ROADTYPE_ROAD);
		passengerBusEngine = getRoadEngine(passengerCargoId);
		mailLorryEngine = getRoadEngine(mailCargoId);
		
		local railList = AIRailTypeList();
		Debug("Rail type count: " + railList.Count());
		current_rail_type = railList.Begin();	// TODO: detect change and perhaps convert rail
		AIRail.SetCurrentRailType(current_rail_type);
		trainEngine = getTrainEngine(current_rail_type);
		passengerCart = getCart(passengerCargoId, current_rail_type);
		mailCart = getCart(mailCargoId, current_rail_type);

		local towns = AITownList();
		towns.Valuate(AITown.GetPopulation);
		towns.Sort(AIAbstractList.SORT_BY_VALUE, false);
		biggestTown = towns.Begin();

		expandMoneyMin = 2 * AIEngine.GetPrice(passengerBusEngine);
		expandMoneyMin = (2 * AIEngine.GetPrice(mailLorryEngine)) < expandMoneyMin ? expandMoneyMin : 2 * AIEngine.GetPrice(mailLorryEngine);
		// OR:
		//expandMoneyMin = AIEngine.GetPrice(trainEngine);
		//expandMoneyMin += 5 * AIEngine.GetPrice(passengerCart);
		//expandMoneyMin += 2 * AIEngine.GetPrice(mailCart);

	}

	function Start() {
		if (!AICompany.SetName("OTVI")) {
			local i = 0;
			while (!AICompany.SetName("OTVI franchise #" + i))
				i++;
		}
		
		local breakdownLevel = AIGameSettings.GetValue("difficulty.vehicle_breakdowns"); 
		if (AIGameSettings.IsValid("difficulty.vehicle_breakdowns") && breakdownLevel != 0) {
			Warning("Breakdowns are on; enabling replacement of vehicles...");
			if (!AICompany.SetAutoRenewStatus(true))
				Error("Couldn't enable auto renew; TODO: implement manual way");
			AICompany.SetAutoRenewMonths(breakdownLevel == 1 ? 12 : 6);
			AICompany.SetAutoRenewMoney(100000); //	this way it shouldn't interfere with normal building
		} else {
			AICompany.SetAutoRenewStatus(false);
		}

		while(passengerBusEngine == -1 && mailLorryEngine == -1) {
			manageLoan();	// no loan (also no hq) for as long as possible
			checkEvents();	// wait for suitable equipment
			
			this.Sleep(1);
		}

		
		// the main control loop
		local date = AIDate.GetCurrentDate();
		local currentYear = AIDate.GetYear(date);
		local currentMonth = AIDate.GetMonth(date);
		local lastYear = currentYear;
		local endYear = currentYear + 10;	// last year to chance search radius for new town connections
		local lastExpansionMonth = currentMonth;	// TODO convert to daycount?
		local hqBuilt = false;
		
		while(true){
			if(currentYear != lastYear && currentYear < endYear) {
				lastYear = currentYear;
				MAX_DISTANCE += 10;
				MIN_DISTANCE -= 1;
			}
			
			if(AICompany.GetMaxLoanAmount() - AICompany.GetLoanAmount() + AICompany.GetBankBalance(AICompany.COMPANY_SELF) > expandMoneyMin && lastExpansionMonth != currentMonth) {
				if(lastMail1 != -1 && lastMail2 != -1) {
					// add buses and mail trucks where there is plenty cargo
					local lorry1 = AIStation.GetStationID(lastMail1);
					local lorry2 = AIStation.GetStationID(lastMail2);
					if(min(AIStation.GetCargoWaiting(lorry1, mailCargoId), AIStation.GetCargoWaiting(lorry2, mailCargoId)) > 3 * AIVehicle.GetCapacity(lastMailVehicle, mailCargoId)) {
						AICompany.SetLoanAmount(AICompany.GetMaxLoanAmount());
						Debug("Adding mailtrucks at " + AIStation.GetName(lorry1) + " to " + AIStation.GetName(lorry2));
						if (connectMailRoadStations(lastMail1, lastMail2))
							lastExpansionMonth = currentMonth;
					}
				}
				if(lastTown1 != -1 && lastTown2 != -1) {
					// add buses and mail trucks where there is plenty cargo
					local pass1 = AIStation.GetStationID(lastTown1);
					local pass2 = AIStation.GetStationID(lastTown2);
					if(min(AIStation.GetCargoWaiting(pass1, passengerCargoId), AIStation.GetCargoWaiting(pass2, passengerCargoId)) > 4 * AIVehicle.GetCapacity(lastPassengerVehicle, passengerCargoId) && lastMaxBusses) {
						AICompany.SetLoanAmount(AICompany.GetMaxLoanAmount());
						Debug("Adding busses at " + AIStation.GetName(pass1) + " to " + AIStation.GetName(pass2));
						if(connectPassengerRoadStations(lastTown1, lastTown2)) {
							lastMaxBusses--;
							lastExpansionMonth = currentMonth;
						}
					}
				}
			}
			
			if(AICompany.GetMaxLoanAmount() - AICompany.GetLoanAmount() + AICompany.GetBankBalance(AICompany.COMPANY_SELF) > 80000 && passengerBusEngine != -1) {
				Debug("We've got enough money to connect stuffs!");
				if(AICompany.GetBankBalance(AICompany.COMPANY_SELF) < 125000)
					AICompany.SetLoanAmount(AICompany.GetMaxLoanAmount());
				//Debug("sub count: " + AISubsidyList().Count() + "; building one: " + subsidyBuilt);
				if (!connect2Towns()) {
					manageLoan();
					Sleep(100);
				}
			}
						
			if (!hqBuilt) {
				buildHQ(biggestTown);
				hqBuilt = true;
			}

			manageLoan();
			
			Sleep(1);
			
			checkEvents();

			date = AIDate.GetCurrentDate();
			currentYear = AIDate.GetYear(date);
			currentMonth = AIDate.GetMonth(date);
		}
	}

	function getTrainEngine(rail_type) {
		local trainEngine_list = AIEngineList(AIVehicle.VT_RAIL);
		trainEngine_list.Valuate(AIEngine.CanRunOnRail, rail_type);
		trainEngine_list.KeepValue(1);

		//trainEngine_list.Valuate(AIEngine.GetReliability);	// dependend on whether breakdowns are on!!
		trainEngine_list.Valuate(AIEngine.GetMaxSpeed);

		trainEngine_list.KeepTop(1);
		local trainEngine = trainEngine_list.Begin();
		Debug("Picked trainEngine: " + AIEngine.GetName(trainEngine));

		return trainEngine;
	}

	function getCart(cargoID, railType) {
		local cart_list = AIEngineList(AIVehicle.VT_RAIL);
		cart_list.Valuate(AIEngine.CanRefitCargo, cargoID);
		cart_list.KeepValue(1);

		cart_list.Valuate(AIEngine.IsWagon);
		cart_list.KeepValue(1);
		
		cart_list.Valuate(AIEngine.CanRunOnRail, railType);
		cart_list.KeepValue(1);

		cart_list.Valuate(AIEngine.GetCapacity)
		cart_list.KeepTop(1);
		
		local cart = cart_list.Begin();
		Debug("Picked cart for cargo " + AICargo.GetCargoLabel(cargoID) + ": " + AIEngine.GetName(cart));

		return cart;
	}
	
	function getRoadEngine(cargoID) {
		local engine_list = AIEngineList(AIVehicle.VT_ROAD);
		engine_list.Valuate(AIEngine.CanRefitCargo, cargoID);
		engine_list.KeepValue(1);

		engine_list.Valuate(AIEngine.GetRoadType);
		engine_list.KeepValue(AIRoad.ROADTYPE_ROAD);

		engine_list.Valuate(AIEngine.GetCapacity)
		engine_list.KeepTop(1);
		if (engine_list.Count() == 0) {
			Warning("No suitable " + AICargo.GetCargoLabel(cargoID) + " engines available (yet?)!!");
			return -1;
		}
		
		local engine = engine_list.Begin();
		Debug("Picked engine for cargo " + AICargo.GetCargoLabel(cargoID) + ": " + AIEngine.GetName(engine));

		return engine;
	}
	
	// 6.3 tiles per day for passengers? // TODO: check or calculate
	function incomeSort(targetTown, startTile, cargoID, engID) {
		local distance = AIMap.DistanceManhattan(startTile, AITown.GetLocation(targetTown));
		local days = distance/7 + 1;	//underestimate
		//Debug(AITown.GetName(targetTown) + " at distance: " + distance + ", days: " + days + " income: " + AICargo.GetCargoIncome(cargoID, distance, days) + " maxProd: " + AITown.GetMaxProduction(targetTown, cargoID));
		return AICargo.GetCargoIncome(cargoID, distance, days) * (AITown.GetLastMonthProduction(targetTown, cargoID) - AITown.GetLastMonthTransported(targetTown, cargoID)) * (365/days);
	}
	
	function stationLocationValuator(tile, length, width) { // TODO check in 4 directions!!!
		local landTiles = AITileList();
		if (AITile.IsBuildableRectangle(tile, length + 1, width)) {
			landTiles.AddRectangle(tile, tile + AIMap.GetTileIndex(length - 1, width - 1));
			landTiles.Valuate(AITile.GetSlope);
			landTiles.KeepValue(AITile.SLOPE_FLAT);
			if (landTiles.Count() == length * width) {
				return AIRail.RAILTRACK_NE_SW;
			}
			// Warning("location only has limited flat tiles in NE_SW dir: " + landTiles.Count());
		}
		if (AITile.IsBuildableRectangle(tile, width, length + 1)) {
			landTiles.AddRectangle(tile, tile + AIMap.GetTileIndex(width - 1, length- 1));
			landTiles.Valuate(AITile.GetSlope);
			landTiles.KeepValue(AITile.SLOPE_FLAT);
			if (landTiles.Count() == length * width) {
				return AIRail.RAILTRACK_NW_SE;
			}
			// Warning("location only has limited flat tiles in NW_SE dir: " + landTiles.Count());
		}
		return 0;
	}
	
	function checkEvents() {
		while (AIEventController.IsEventWaiting()) {
			local e = AIEventController.GetNextEvent();
			//Debug("Got an eventA: " + e);
			switch (e.GetEventType()) {
				case AIEvent.AI_ET_VEHICLE_CRASHED:
					local ec = AIEventVehicleCrashed.Convert(e);
					local v  = ec.GetVehicleID();
					Debug("We have a crashed vehicle (" + v + ")");
					// TODO: Handle the crashed vehicle; aka: clone?
					break;
				case AIEvent.AI_ET_ENGINE_AVAILABLE:
					Debug("New engine available; rechecking current engines...");
					local railList = AIRailTypeList();
					local new_type = railList.Begin();	// TODO
					if (new_type != current_rail_type) {
						Warning("New rail type!");
						for (local i = 0; i < 10; i++) {
							if (AIRail.ConvertRailType(AIMap.GetTileIndex(0,0), AIMap.GetTileIndex(AIMap.GetMapSizeX(), AIMap.GetMapSizeY()), new_type)) {
								Debug("Rail conversion success");
							} else {
								Error("At least partial failure converting rail!");
							}
							Sleep(25);
						}
						current_rail_type = new_type;
					}
					AIRail.SetCurrentRailType(current_rail_type);
					trainEngine = getTrainEngine(current_rail_type);
					passengerCart = getCart(passengerCargoId, current_rail_type);
					mailCart = getCart(mailCargoId, current_rail_type);
					passengerBusEngine = getRoadEngine(passengerCargoId);
					mailLorryEngine = getRoadEngine(mailCargoId);

					expandMoneyMin = 2 * AIEngine.GetPrice(passengerBusEngine);
					expandMoneyMin = (2 * AIEngine.GetPrice(mailLorryEngine)) < expandMoneyMin ? expandMoneyMin : 2 * AIEngine.GetPrice(mailLorryEngine);
					break;
				case AIEvent.AI_ET_VEHICLE_LOST:
					Debug("A vehicle got lost!!!");
					// TODO: nuke vehicle??
					break;
				case AIEvent.AI_ET_VEHICLE_WAITING_IN_DEPOT:
					Debug("A vehicle arrived in the depot for selling");
					local ec = AIEventVehicleWaitingInDepot.Convert(e);
					local v  = ec.GetVehicleID();
					AIVehicle.SellVehicle(v);
					break;
				case AIEvent. AI_ET_VEHICLE_UNPROFITABLE:
					Debug("Unprofitable vehicle detected; sending to depot");
					// TODO: evaluate on one more year?
					local ec = AIEventVehicleUnprofitable.Convert(e);
					local v  = ec.GetVehicleID();
					AIVehicle.SendVehicleToDepot(v);
					break;
				case AIEvent.AI_ET_SUBSIDY_OFFER:
					local ec = AIEventSubsidyOffer.Convert(e);
					local s = ec.GetSubsidyID();
					Debug("New subsidy detected: " + s);
					if (AICompany.GetMaxLoanAmount() - AICompany.GetLoanAmount() + AICompany.GetBankBalance(AICompany.COMPANY_SELF) > 70000) {
						Debug("We've got enough money to get the subsidized route!");
						if (AICompany.GetBankBalance(AICompany.COMPANY_SELF) < 125000)
							AICompany.SetLoanAmount(AICompany.GetMaxLoanAmount());
						connectSubsidy(s);
					} else {
						Warning("Not enough money to get this one now.");
					}
					break;
				case AIEvent.AI_ET_SUBSIDY_AWARDED:
					subsidyBuilt = false;
					local ec = AIEventSubsidyAwarded.Convert(e);
					local s = ec.GetSubsidyID();
					if (AICompany.IsMine(AISubsidy.GetAwardedTo(s)))
						Debug("We got the subsidy!!!!");
					else
						Warning("!!! We failed to win the subsidy!!!!");
					break;
				case AIEvent.AI_ET_SUBSIDY_OFFER_EXPIRED:
					subsidyBuilt = false;
					Debug("A subsidy expired");
					break;
				default:
					//Debug("Unhandled event:" + e);
			}
		}
	}
	
	function manageLoan() {
		//repay loan
		local balance = AICompany.GetBankBalance(AICompany.COMPANY_SELF);
		local balanceMod = (balance / AICompany.GetLoanInterval()) * AICompany.GetLoanInterval();
		local loan = AICompany.GetLoanAmount();
		
		if (balance < 0) {
			// not good... we lost money?
			AICompany.SetLoanAmount(AICompany.GetMaxLoanAmount());
			balance = AICompany.GetBankBalance(AICompany.COMPANY_SELF);
			balanceMod = (balance / AICompany.GetLoanInterval()) * AICompany.GetLoanInterval();
			loan = AICompany.GetLoanAmount();
		}
		
		if (loan > 0 && balanceMod > 0) {
			if (balanceMod > 100000)
				balanceMod = 100000;
			AICompany.SetLoanAmount(loan - balanceMod);
		}
	}

	/**
	 * 2 passenger stations build, make road and then place busses
	 */
	function connectTownsByRoad(starts, goals) {
		roadFinder.cost.max_cost = 15 * AIMap.DistanceManhattan(starts[0], goals[0]);
		roadFinder.InitializePath(starts, goals);
		local path = false;
		local i = 0;
		local err;
		//local iterations = 300 * AIMap.DistanceManhattan(starts[0], goals[0]); // if we pass this; something is wrong
		local iterations = -1;	// TODO limit to reasonable amount, elevate if fails
		local startTick = GetTick();
		// TODO check whether higher values helps finding faster
		Debug("pathfinding for max " + roadFinder.cost.max_cost + " cost and iterations: " + iterations);
		path = roadFinder.FindPath(iterations);
		Debug("Total ticks passed: " + (GetTick() - startTick));

		if(!path) {
			if (path == false) {
				Error("!! No path found, too little iterations...");	// impossible atm
				roadFinder.reset();
				return false;
			} else {
				Warning("!! No path found, too expensive, retrying...");
				roadFinder.reset();
				roadFinder.cost.max_cost = 30 * AIMap.DistanceManhattan(starts[0], goals[0]);
				Debug("pathfinding for max " + roadFinder.cost.max_cost + " cost and iterations: " + iterations);
				roadFinder.InitializePath(starts, goals);
				path = roadFinder.FindPath(iterations);
			}
		}
		if(!path) {
			roadFinder.reset();
			Error("Impossible to create route, way too expensive");
			return false;
		}
		Debug("path found for cost: " + path.GetCost() + ", building");

		while (path != null) {
			local par = path.GetParent();
			if (par != null) {
				local last_node = path.GetTile();
				if (AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) == 1 ) {
					if (!AIRoad.BuildRoad(path.GetTile(), par.GetTile())) {
						switch(AIError.GetLastError()) {
							case AIError.ERR_VEHICLE_IN_THE_WAY:
								Warning("Building road failed temporary: vehicle in the way...");
								// try again till the stupid vehicle moved?
								while(!AIRoad.BuildRoad(path.GetTile(), par.GetTile()));
								break;
							case AIRoad.ERR_ROAD_WORKS_IN_PROGRESS:
								Warning("Building road failed: road works in progress");
								// good, someone else is building this, skip tile
								break;
							case AIError.ERR_ALREADY_BUILT:
								// Debug("Building road failed: already built");
								// even better; someone else already built this; silent ignore
								break;
							case AIError.ERR_LAND_SLOPED_WRONG:
								Error("Building road failed: wrong land slope");
								// not good; redo from start or terraform?
								return connectTownsByRoad(starts, goals);
							case AIError.ERR_AREA_NOT_CLEAR:
								Warning("Building road failed: not clear, demolishing tile...");
								AISign.BuildSign(path.GetTile(), "I'm on your land, demolishing your stuffs");
								// demolish and retry?
								if (AITile.DemolishTile(path.GetTile())) {
									AIRoad.BuildRoad(path.GetTile(), par.GetTile());
								} else {	// happens when other ai builds after calc
									return connectTownsByRoad(starts, goals);
								}
								break;
							case AIRoad.ERR_ROAD_ONE_WAY_ROADS_CANNOT_HAVE_JUNCTIONS:
								Error("Building road failed: no junctions on one way roads allowed");
								// can't happen? Not building those
								return false;
							case AIError.ERR_NOT_ENOUGH_CASH:
								Error("Panic: Out of cash!!");
								return false;
							default:
								Warning("Unknown error during road building " + AIError.GetLastErrorString());
								break;	// no clue what this is; silent ignore for now
						}
					}
				} else {
					/* Build a bridge or tunnel. */
					if (!AIBridge.IsBridgeTile(path.GetTile()) && !AITunnel.IsTunnelTile(path.GetTile())) {
						/* If it was a road tile, demolish it first. Do this to work around expended roadbits. */
						if (AIRoad.IsRoadTile(path.GetTile())) AITile.DemolishTile(path.GetTile());
						if (AITunnel.GetOtherTunnelEnd(path.GetTile()) == par.GetTile()) {
							if (!AITunnel.BuildTunnel(AIVehicle.VT_ROAD, path.GetTile())) {
								/* An error occured while building a tunnel. TODO: handle it. */
								// TODO expected to fail once per tunnel atm
								// AISign.BuildSign(path.GetTile(), ("Tunnel fail"));
								Error("Couldn't build tunnel (normal once per tunnel atm");
							}
						} else {
							local bridge_list = AIBridgeList_Length(AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) + 1);
							bridge_list.Valuate(AIBridge.GetMaxSpeed);
							bridge_list.Sort(AIAbstractList.SORT_BY_VALUE, false);
							if (!AIBridge.BuildBridge(AIVehicle.VT_ROAD, bridge_list.Begin(), path.GetTile(), par.GetTile())) {
								/* An error occured while building a bridge. TODO: handle it. */
								// TODO expected to fail once per bridge atm
								// AISign.BuildSign(path.GetTile(), ("Bridge fail"));
								Error("Couldn't build bridge (normal once per bridge atm)");
							}
						}
					}
				}
			}
			path = par;
		}

		/*while (path != null) {
			if (path.GetParent() != null) {
				local par = path.GetParent().GetTile();
				if (!AIRoad.AreRoadTilesConnected(path.GetTile(), par) && (!AIBridge.IsBridgeTile(path.GetTile()) || AIBridge.GetOtherBridgeEnd(path.GetTile()) != par)) {
					AIRoad.BuildRoad(path.GetTile(), par);
				}
			}
			path = path.GetParent();
		}*/
		Debug("path done");
		return true;
	}

	function connectPassengerRoadStations(station1, station2) {		
		local bus1 = AIVehicle.BuildVehicle(myStationsTileList.GetValue(station1), passengerBusEngine);
		if (!AIVehicle.IsValidVehicle(bus1))
			return false;
		if(!AIVehicle.RefitVehicle(bus1, passengerCargoId)) {
			Error("Error refitting bus: " + AIError.GetLastErrorString());
			return false;
		}

		Debug("Built bus: " + bus1);
		lastPassengerVehicle = bus1;
		AIOrder.AppendOrder(bus1, station1, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(bus1, station2, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIVehicle.StartStopVehicle(bus1);
		local bus2 = AIVehicle.BuildVehicle(myStationsTileList.GetValue(station2), passengerBusEngine);
		if (!AIVehicle.IsValidVehicle(bus2))
			return false; 	// TODO change to return 1 for 1 success
		if(!AIVehicle.RefitVehicle(bus2, passengerCargoId)) {
			Error("Error refitting bus: " + AIError.GetLastErrorString());
			return false; // TODO
		}
		Debug("Built bus: " + bus2);
		AIOrder.AppendOrder(bus2, station2, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(bus2, station1, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIVehicle.StartStopVehicle(bus2);

		return true;	// TODO change to 2
	}
	
	function connectMailRoadStations(lorry1, lorry2) {
		local mail1 = AIVehicle.BuildVehicle(myLorryTileList.GetValue(lorry1), mailLorryEngine);
		if(!AIVehicle.IsValidVehicle(mail1))
			return false;

		Debug("Built a mail truck: " + mail1 + " at location " + lorry1);
		lastMailVehicle = mail1;
		AIOrder.AppendOrder(mail1, lorry1, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(mail1, lorry2, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIVehicle.StartStopVehicle(mail1);
		local mail2 = AIVehicle.BuildVehicle(myLorryTileList.GetValue(lorry2), mailLorryEngine);
		if(!AIVehicle.IsValidVehicle(mail2))
			return false;

		Debug("Built a mail truck: " + mail2 + " at location " + lorry2);
		AIOrder.AppendOrder(mail2, lorry2, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(mail2, lorry1, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIVehicle.StartStopVehicle(mail2);

		return true;
	}
	
	function getRoadTile(t) {
		local adjacent = AITileList();
		adjacent.AddTile(t - AIMap.GetTileIndex(1,0));
		adjacent.AddTile(t - AIMap.GetTileIndex(0,1));
		adjacent.AddTile(t - AIMap.GetTileIndex(-1,0));
		adjacent.AddTile(t - AIMap.GetTileIndex(0,-1));
		adjacent.Valuate(AIRoad.IsRoadTile);
		adjacent.KeepValue(1);
		adjacent.Valuate(AIRoad.IsRoadDepotTile);
		adjacent.KeepValue(0);
		adjacent.Valuate(AIRoad.IsRoadStationTile);
		adjacent.KeepValue(0);
		if (adjacent.Count())
			return adjacent.Begin();
		else
			return null;
	}

	/**
	 * hack to support basic sqrt function for other functions
	 * TODO: consider using lookup table for speed reasons (as suggested by Truebrain)
	 * TODO: useless now?
	 */
/*	function sqrt(i) {
		if (i == 0)
			return 0;   // Avoid divide by zero
		local n = (i / 2) + 1;       // Initial estimate, never low
		local n1 = (n + (i / n)) / 2;
		while (n1 < n) {
			n = n1;
			n1 = (n + (i / n)) / 2;
		}
		return n;
	}*/

	function buildNonInlinePassengerStation(town) {
		Debug("Building non inline station...");
		// Find empty square as close to town centre as possible
		local spotFound = false;
		local curRange = 1;
		local maxRange = sqrt(AITown.GetPopulation(town)/100) + 4; // search larger area
		local area = AITileList();

		while (curRange < maxRange) {
			//Debug("looking with range " + curRange);
			area.AddRectangle(AITown.GetLocation(town) - AIMap.GetTileIndex(curRange, curRange), AITown.GetLocation(town) + AIMap.GetTileIndex(curRange, curRange));
			area.Valuate(AITile.IsBuildable);
			area.KeepValue(1);
			if (area.Count()) {
				//Debug("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					local opening = getRoadTile(t);
					if(opening) {
						// Debug("tile " + t + " has opening " + opening);
						if (AIRoad.BuildRoad(opening, t) && AIRoad.BuildRoadStation(t, opening, AIRoad.ROADVEHTYPE_BUS, AIStation.STATION_JOIN_ADJACENT)) {
							//AISign.BuildSign(t, ("bus " + AITown.GetName(town)));
							return t;
						}
						Error("Error building passenger station " + AIError.GetLastErrorString());
						//AISign.BuildSign(t, "" + AIRoad.GetNeighbourRoadCount(t));
					} //else {
						// Debug("tile" + t + " has no opening");
					//}
				}
				curRange++;
			} else {
				curRange++;
			}
		}
		Debug("No possible station location found");
		return false;
	}
	
	function buildPassengerStation(town) {
		Debug("Building station...");
		// Find empty square as close to town centre as possible
		local spotFound = false;
		local curRange = 1;
		local maxRange = sqrt(AITown.GetPopulation(town)/100) + 2; //TODO check value correctness 
		local area = AITileList();
		
		while (curRange < maxRange) {
			//Debug("looking with range " + curRange);
			area.AddRectangle(AITown.GetLocation(town) - AIMap.GetTileIndex(curRange, curRange), AITown.GetLocation(town) + AIMap.GetTileIndex(curRange, curRange));
			area.Valuate(AIRoad.IsRoadTile);
			area.KeepValue(1);
			area.Valuate(AIRoad.IsDriveThroughRoadStationTile);
			area.KeepValue(0);
			area.Valuate(AIRoad.GetNeighbourRoadCount);
			area.KeepValue(2);	// entrance and exit; allow 1 as well?
			if (area.Count()) {
				//Debug("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					local opening = getRoadTile(t);
					if(opening) {
						// Debug("tile " + t + " has opening " + opening);
						if (/*AIRoad.BuildRoad(opening, t) &&*/ AIRoad.BuildDriveThroughRoadStation(t, opening, AIRoad.ROADVEHTYPE_BUS, AIStation.STATION_JOIN_ADJACENT)) {
							//AISign.BuildSign(t, ("bus " + AITown.GetName(town)));
							return t;
						}
						if(AIError.GetLastError() == AIRoad.ERR_ROAD_CANNOT_BUILD_ON_TOWN_ROAD)
							return buildNonInlinePassengerStation(town);
						Error("Error building passenger station " + AIError.GetLastErrorString());
						//AISign.BuildSign(t, AIError.GetLastErrorString());

					} //else {
						// Debug("tile" + t + " has no opening");
					//}
				}
				curRange++;
			} else {
				curRange++;
			}
		}
		Debug("No possible station location found");
		return false;
	}
	
	function buildLorryStation(location) {
		Debug("Building lorry station...");
		// Find empty square as close to tile as possible
		local spotFound = false;
		local curRange = 1;
		local maxRange = 4; //TODO check value correctness
		local area = AITileList();
		local area2 = AITileList();

		// We got the north most tile; I wanna search around it's center
		location = location + AIMap.GetTileIndex(1, 1);
		
		while (curRange < maxRange) {
			//AILog.Info("looking with range " + curRange);
			area.AddRectangle(location - AIMap.GetTileIndex(curRange, curRange), location + AIMap.GetTileIndex(curRange, curRange));
			area.Valuate(AITile.IsBuildable);
			area.KeepValue(1);
			area2.AddRectangle(location - AIMap.GetTileIndex(curRange + 1, curRange + 1), location + AIMap.GetTileIndex(curRange + 1, curRange + 1));
			area2.Valuate(AITile.IsBuildable);
			area2.KeepValue(1);
			if (area.Count()) {
				//AILog.Info("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					local opening = area2.Begin();
					while ((AIMap.DistanceManhattan(t, opening) != 1 || opening == t) && area2.HasNext()) {
						opening = area2.Next();
					}
					if(opening) {
						// AILog.Info("tile " + t + " has opening " + opening);
						if (AIRoad.BuildRoad(opening, t) && AIRoad.BuildDriveThroughRoadStation(t, opening, AIRoad.ROADVEHTYPE_TRUCK, AIStation.STATION_JOIN_ADJACENT)) {
							//AISign.BuildSign(t, ("Mail " + AITown.GetName(town)));
							return t;
						}
						//AILog.Info("tile " + t + " has wrong slope!!!");
					} //else {
						// AILog.Info("tile" + t + " has no opening");
					//}
				}
				curRange++;
			} else {
				curRange++;
			}
		}
		AILog.Info("No possible lorry station location found");
		return null;
	}
		
	function buildMailStation(town) {	// TODO check mail acceptance
		if (mailLorryEngine == -1) {
			Warning("No mail engine available, not building mail stations");
			return null;
		}

		Debug("Building mail station...");
		// Find empty square as close to town centre as possible
		local spotFound = false;
		local curRange = 1;
		local maxRange = sqrt(AITown.GetPopulation(town)/100) + 1; //TODO check value correctness
		local area = AITileList();
		
		while (curRange < maxRange) {
			//Debug("looking with range " + curRange);
			area.AddRectangle(AITown.GetLocation(town) - AIMap.GetTileIndex(curRange, curRange), AITown.GetLocation(town) + AIMap.GetTileIndex(curRange, curRange));
			area.Valuate(AITile.IsBuildable);
			area.KeepValue(1);
			if (area.Count()) {
				//Debug("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					local opening = getRoadTile(t);
					if(opening) {
						// Debug("tile " + t + " has opening " + opening);
						if (AIRoad.BuildRoad(opening, t) && AIRoad.BuildDriveThroughRoadStation(t, opening, AIRoad.ROADVEHTYPE_TRUCK, AIStation.STATION_JOIN_ADJACENT)) {
							//AISign.BuildSign(t, ("Mail " + AITown.GetName(town)));
							return t;
						}
						//Debug("tile " + t + " has wrong slope!!!");
					} //else {
						// Debug("tile" + t + " has no opening");
					//}
				}
				curRange++;
			} else {
				curRange++;
			}
		}
		Debug("No possible mail location found");
		return null;
	}
	
	function buildHQ(town) {
		Debug("building HQ...");
		// Find empty 2x2 square as close to town centre as possible
		local spotFound = false;
		local curRange = 1;
		local maxRange = sqrt(AITown.GetPopulation(town)/100) + 5; //TODO check value correctness
		local area = AITileList();
		
		while (curRange < maxRange) {
			area.AddRectangle(AITown.GetLocation(town) - AIMap.GetTileIndex(curRange, curRange), AITown.GetLocation(town) + AIMap.GetTileIndex(curRange, curRange));
			area.Valuate(AITile.IsBuildableRectangle, 2, 2);
			area.KeepValue(1);
			if (area.Count()) {
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					local opening = getRoadTile(t);
					if(opening) {
						if (AICompany.BuildCompanyHQ(t)) {
							// TODO different name if franchise
							AISign.BuildSign(t, ("HQ OTVI"));
							return t;
						}
					}
				}
				curRange++;
			} else {
				curRange++;
			}
		}
		Debug("No possible HQ location found");
		return null;
	}

	function buildRoadDepot(station) {
		Debug("Building depot...");
		// Find empty square as close to station as possible
		local spotFound = false;
		local curRange = 1;
		local area = AITileList();
		
		while (curRange < 10) {
			//AILog.Info("looking with range " + curRange);
			area.AddRectangle(station - AIMap.GetTileIndex(curRange, curRange), station + AIMap.GetTileIndex(curRange, curRange));
			area.Valuate(AITile.IsBuildable);
			area.KeepValue(1);
			if (area.Count()) {
				//AILog.Info("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					local opening = getRoadTile(t);
					if(opening) {
						//AILog.Info("tile " + t + " has opening " + opening);
						if (AIRoad.BuildRoad(t, opening) && AIRoad.BuildRoadDepot(t, opening)) {
							//AISign.BuildSign(t, ("depot"));
							return t;
						}
						//AILog.Info("tile " + t + " has wrong slope!!!");
					} //else {
						//AILog.Info("tile" + t + " has no opening");
					//}
				}
				curRange++; // the found options had no road connections; enlarge search area
			} else {
				curRange++;
			}
		}
		AILog.Info("No possible depot location found");
		return null;
	}
	
	function buildTownRoadDepot(station, town) {
		Debug("Building town depot...");
		// Find empty square as close to station as possible
		local spotFound = false;
		local curRange = 1;
		local maxRange = sqrt(AITown.GetPopulation(town)/100) + 4; //TODO check value correctness
		local area = AITileList();
		
		while (curRange < maxRange) {
			//Debug("looking with range " + curRange);
			area.AddRectangle(station - AIMap.GetTileIndex(curRange, curRange), station + AIMap.GetTileIndex(curRange, curRange));
			area.Valuate(AITile.IsBuildable);
			area.KeepValue(1);
			if (area.Count()) {
				//Debug("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					local opening = getRoadTile(t);
					if(opening) {
						//Debug("tile " + t + " has opening " + opening);
						if (AIRoad.BuildRoad(t, opening) && AIRoad.BuildRoadDepot(t, opening)) {
							//AISign.BuildSign(t, ("depot"));
							return t;
						}
						//Debug("tile " + t + " has wrong slope!!!");
					} //else {
						//Debug("tile" + t + " has no opening");
					//}
				}
				curRange++; // the found options had no road connections; enlarge search area
			} else {
				curRange++;
			}
		}
		Debug("No possible depot location found");
		return null;
	}
	
	function connectSubIndustriesByRoad(subID) {
		local station1 = buildLorryStation(AIIndustry.GetLocation(AISubsidy.GetSourceIndex(subID)));
		if (!station1)
			return false; // bugger
		local loc2;
		if (AISubsidy.GetDestinationType(subID) == AISubsidy.SPT_TOWN)
			loc2 = AITown.GetLocation(AISubsidy.GetDestinationIndex(subID));
		else
			loc2 = AIIndustry.GetLocation(AISubsidy.GetDestinationIndex(subID));
		local station2 = buildLorryStation(loc2);
		if (!station2)
			return false; // bugger

		Debug("Route between 2 industries is " + AITile.GetDistanceManhattanToTile(station1, station2));
		local starts = [station1];
		local goals = [station2];
		if(!connectTownsByRoad(starts, goals))
			return false;

		local depot1 = buildRoadDepot(station1);
		myStationsTileList.AddItem(station1, depot1);
		local depot2 = buildRoadDepot(station2); // TODO build town depot if destination is town
		myStationsTileList.AddItem(station2, depot2);

		if(!depot1 && !depot2)
			return false;

		//connectTownsByRoad(station1, depot1);
		//connectTownsByRoad(station2, depot2);
		connectIndustryRoadStations(station1, station2, AISubsidy.GetCargoType(subID));
	}

	function connectIndustryRoadStations(station1, station2, cargoID) {	
		local engine = getRoadEngine(cargoID);
		local count = (AITile.GetDistanceManhattanToTile(station1, station2) / 10) + 1;
		for (local i = 0; i < count; i++) {
			local vehicle = AIVehicle.BuildVehicle(myStationsTileList.GetValue(station1), engine);
			if(!AIVehicle.RefitVehicle(vehicle, cargoID)) {
				Error("Error refitting vehicle: " + AIError.GetLastErrorString());
				return false;
			}
			Debug("Built truck " + vehicle + " which is valid: " + AIVehicle.IsValidVehicle(vehicle));
			AIOrder.AppendOrder(vehicle, station1, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
			AIOrder.AppendOrder(vehicle, station2, AIOrder.AIOF_NONE | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
			AIVehicle.StartStopVehicle(vehicle);
			if (!AIVehicle.IsValidVehicle(vehicle))
				return false;
		}
		return true;
	}

	function connectSubTownsByRoad(subID) {
		biggestTown = AISubsidy.GetSourceIndex(subID);

		Debug("First subsidy town: " + AITown.GetName(biggestTown));
		local station1 = buildPassengerStation(biggestTown);
		if(!station1) {
			Error("!!! Building station failed !!!");
			return false;
		}

		local mail1;

		local depot1 = myTownList.GetValue(biggestTown);
		if (!depot1) {
			mail1 = buildMailStation(biggestTown);
			depot1 = buildTownRoadDepot(station1, biggestTown);
			if(!depot1) {
				// to prevent it from choosing this "bad" town again
				myTownList.SetValue(biggestTown, -1);
				return false;
			}
			myTownList.SetValue(biggestTown, depot1);
			if (mail1) { 
				myLorryTileList.AddItem(mail1, depot1);
				myDepotsTileList.AddItem(depot1, mail1);
			}
		} else {
			mail1 = myDepotsTileList.GetValue(myTownList.GetValue(biggestTown));
			if (!mail1) {
				mail1 = buildMailStation(biggestTown);
				if (mail1) { 
					myLorryTileList.AddItem(mail1, depot1);
					myDepotsTileList.AddItem(depot1, mail1);
				}
			}
		}
		myStationsTileList.AddItem(station1, depot1);

		local connectTown = AISubsidy.GetDestinationIndex(subID);

		Debug("Connecting subsidy town: " + AITown.GetName(connectTown) + " at a manhattan distance of " + AITile.GetDistanceManhattanToTile(AITown.GetLocation(biggestTown), AITown.GetLocation(connectTown)));

		local station2 = buildPassengerStation(connectTown);
		if(!station2) {
			Error("!!! Building station failed !!!");
			return false;
		}

		local mail2;

		local depot2 = myTownList.GetValue(connectTown);
		if (!depot2) {
			if(mail1)
				mail2 = buildMailStation(connectTown);
			depot2 = buildTownRoadDepot(station2, connectTown);
			if(!depot2)
				return false;
			myTownList.SetValue(connectTown, depot2);
			if (mail2) { 
				myLorryTileList.AddItem(mail2, depot2);
				myDepotsTileList.AddItem(depot2, mail2);
			}
		} else {
			mail2 = myDepotsTileList.GetValue(myTownList.GetValue(connectTown));
			if (!mail2 && mail1) {
				if(mail1)
					mail2 = buildMailStation(connectTown);
				if (mail2) { 
					myLorryTileList.AddItem(mail2, depot2);
					myDepotsTileList.AddItem(depot2, mail2);
				}
			}
		}
		myStationsTileList.AddItem(station2, depot2);

		local distance = AITile.GetDistanceManhattanToTile(station1, station2);
		Debug("Route between 2 cities is " + distance);
		local opening1 = AIRoad.GetRoadDepotFrontTile(depot1);
		local opening2 = AIRoad.GetRoadDepotFrontTile(depot2);
		local starts = mail1 ? [opening1, mail1, station1] : [opening1, station1];
		local goals = mail2 ? [opening2, mail2, station2] : [opening2, station2];
		if(!connectTownsByRoad(starts, goals))
			return false;
		if (mail1 && mail2) {
			// make sure the mail stations are connected to the road build for the passengers
			//connectTownsByRoad(station1, mail1);
			//connectTownsByRoad(station2, mail2);
			// sending out mail truck first, otherwise it might get stopped at a drive through station by our own bus
			lastMail1 = mail1;
			lastMail2 = mail2;
			connectMailRoadStations(mail1, mail2);
		}

		//we picked BIG towns, lets try to build more busses in 1 go
		local pop1 = AITown.GetPopulation(biggestTown);
		local pop2 = AITown.GetPopulation(connectTown);
		local engine = getRoadEngine(passengerCargoId);
		// TODO: see whether this value max sense:
		local divider = AIEngine.GetCapacity(engine) * 5;
		local maxBussesPop = (pop1 < pop2 ? pop1 : pop2) / divider;	// built a bus for every hundred passengers in the smallest town
		local maxBussesDist = (distance / (AIEngine.GetMaxSpeed(engine) / 10));	// tweaked to 10. May need to be raised to 12...
		local maxBusses = maxBussesPop < maxBussesDist ? maxBussesPop : maxBussesDist;
		maxBusses++;	// plus 2 busses for both stops
		Debug("Want to deploy " + (maxBusses * 2) + " busses for this route (pop/dist:) " + ((maxBussesPop + 1) * 2) + "/" + ((maxBussesDist + 1) * 2));
		while(maxBusses-- && AICompany.GetBankBalance(AICompany.COMPANY_SELF) > 12500)	// actually 12.500 is only enough for 1 bus
			connectPassengerRoadStations(station1, station2);
		if(maxBusses) {
			lastMaxBusses = maxBusses;
			lastTown1 = station1;
			lastTown2 = station2;
		}
	}

	function connect2Towns() {
		// look for biggest towns
		local townlist = AITownList();
		townlist.Valuate(AITown.GetLastMonthProduction, mailCargoId);
		townlist.Sort(AIAbstractList.SORT_BY_VALUE, false);
		biggestTown = townlist.Begin();
		
		// skip towns already taken
		while (myTownList.GetValue(biggestTown)) {
			if(!townlist.HasNext()) {
				Debug("No more towns left");
				manageLoan();
				Sleep(100);
				return;
			}
			//Debug("Skipping town (already taken): " + AITown.GetName(biggestTown));
			biggestTown = townlist.Next();
		}
		
		Debug("biggest town: " + AITown.GetName(biggestTown));
		local bigLoc = AITown.GetLocation(biggestTown);
		// find connecting town between MIN and MAX manhattan distance
		townlist = AITownList();
		townlist.Valuate(incomeSort, bigLoc, mailCargoId, trainEngine);
		townlist.Sort(AIAbstractList.SORT_BY_VALUE, false);
		local connectTown = townlist.Begin();
		local distance = AITile.GetDistanceManhattanToTile(bigLoc, AITown.GetLocation(connectTown));
		while (distance > MAX_DISTANCE || distance < MIN_DISTANCE) {
			Debug("skipping town: " + AITown.GetName(connectTown) + " at distance " + AITile.GetDistanceManhattanToTile(bigLoc, AITown.GetLocation(connectTown)) + " value: " + townlist.GetValue(connectTown));
			if (!townlist.HasNext()) {	// TODO: add check like: OR skipped 20 or value < maxvalue/4 or something
				Debug("Out of towns");
				return false; // too bad, out of towns
			}
			connectTown = townlist.Next();
			distance = AITile.GetDistanceManhattanToTile(bigLoc, AITown.GetLocation(connectTown));
		}
		
		Debug("Connecting town: " + AITown.GetName(connectTown) + " at a manhattan distance of " + AITile.GetDistanceManhattanToTile(bigLoc, AITown.GetLocation(connectTown)) + " value: " + townlist.GetValue(connectTown));

		if (AITown.GetPopulation(biggestTown) < 1000 || AITown.GetPopulation(connectTown) < 1000)
			return connect2TownsByRoad(biggestTown, connectTown);
		else {
			if(connect2TownsByRail(biggestTown, connectTown) == false) {
				Warning("Train failed for this route, trying busses instead...");
				return connect2TownsByRoad(biggestTown, connectTown);	// try road if rail fails
			}
		}
	}

	function connect2TownsByRoad(biggestTown, connectTown) {
		local station1 = buildPassengerStation(biggestTown);
		local mail1;
		if (!station1) {
			Warning("!!! Building station failed !!!");
			return false;
		}
		local depot1 = myTownList.GetValue(biggestTown);
		if (!depot1) {
			mail1 = buildMailStation(biggestTown);
			depot1 = buildTownRoadDepot(station1, biggestTown);
			if(!depot1) {
				// to prevent it from choosing this "bad" town again
				myTownList.SetValue(biggestTown, -1);
				return false;
			}
			myTownList.SetValue(biggestTown, depot1);
			if (mail1) { 
				myLorryTileList.AddItem(mail1, depot1);
				myDepotsTileList.AddItem(depot1, mail1);
			}
		} else {
			mail1 = myDepotsTileList.GetValue(myTownList.GetValue(biggestTown));
			if (!mail1) {
				mail1 = buildMailStation(biggestTown);
				if (mail1) { 
					myLorryTileList.AddItem(mail1, depot1);
					myDepotsTileList.AddItem(depot1, mail1);
				}
			}
		}
		myStationsTileList.AddItem(station1, depot1);

		local station2 = buildPassengerStation(connectTown);
		if (!station2) {
			Warning("!!! Building station failed !!!");
			return false;
		}

		local mail2;

		local depot2 = myTownList.GetValue(connectTown);
		if (!depot2) {
			if(mail1)
				mail2 = buildMailStation(connectTown);
			depot2 = buildTownRoadDepot(station2, connectTown);
			if(!depot2)
				return false;
			myTownList.SetValue(connectTown, depot2);
			if (mail2) { 
				myLorryTileList.AddItem(mail2, depot2);
				myDepotsTileList.AddItem(depot2, mail2);
			}
		} else {
			mail2 = myDepotsTileList.GetValue(myTownList.GetValue(connectTown));
			if (!mail2 && mail1) {
				if(mail1)
					mail2 = buildMailStation(connectTown);
				if (mail2) { 
					myLorryTileList.AddItem(mail2, depot2);
					myDepotsTileList.AddItem(depot2, mail2);
				}
			}
		}
		myStationsTileList.AddItem(station2, depot2);

		local distance = AITile.GetDistanceManhattanToTile(station1, station2);
		Debug("Route between 2 cities is " + distance);
		local opening1 = AIRoad.GetRoadDepotFrontTile(depot1);
		local opening2 = AIRoad.GetRoadDepotFrontTile(depot2);
		local starts = mail1 ? [opening1, mail1, station1] : [opening1, station1];
		local goals = mail2 ? [opening2, mail2, station2] : [opening2, station2];
		if(!connectTownsByRoad(starts, goals)) {
			if(!connectTownsByRoad(starts, goals)) {	// Try once more if it failed
				return false;
			}
		}
		if (mail1 && mail2) {
			// make sure the mail stations are connected to the road build for the passengers
			//connectTownsByRoad(station1, mail1);
			//connectTownsByRoad(station2, mail2);
			// sending out mail truck first, otherwise it might get stopped at a drive through station by our own bus
			lastMail1 = mail1;
			lastMail2 = mail2;
			connectMailRoadStations(mail1, mail2);
		}

		//we picked BIG towns, lets try to build more busses in 1 go
		local pop1 = AITown.GetPopulation(biggestTown);
		local pop2 = AITown.GetPopulation(connectTown);
		local engine = getRoadEngine(passengerCargoId);
		// TODO: see whether this value max sense:
		local divider = AIEngine.GetCapacity(engine) * 5;
		local maxBussesPop = (pop1 < pop2 ? pop1 : pop2) / divider;	// built a bus for every hundred passengers in the smallest town
		local maxBussesDist = (distance / (AIEngine.GetMaxSpeed(engine) / 10));	// tweaked to 10. May need to be raised to 12...
		local maxBusses = maxBussesPop < maxBussesDist ? maxBussesPop : maxBussesDist;
		maxBusses++;	// plus 2 busses for both stops
		Debug("Want to deploy " + (maxBusses * 2) + " busses for this route (pop/dist:) " + ((maxBussesPop + 1) * 2) + "/" + ((maxBussesDist + 1) * 2));
		while(maxBusses-- && AICompany.GetBankBalance(AICompany.COMPANY_SELF) > 12500)	// actually 12.500 is only enough for 1 bus
			connectPassengerRoadStations(station1, station2);
		if(maxBusses) {
			lastMaxBusses = maxBusses;
			lastTown1 = station1;
			lastTown2 = station2;
		}

		return true;
	}

	function connectSubIndustriesByTrain(subID) {
		// TODO: length station = f(distance, cartSize, production);
		local length = 4;
		local station1 = buildTrainStation(AIIndustry.GetLocation(AISubsidy.GetSourceIndex(subID)), length);
		if (!station1)
			return false; // bugger
		local loc2;
		if (AISubsidy.GetDestinationType(subID) == AISubsidy.SPT_TOWN)
			loc2 = AITown.GetLocation(AISubsidy.GetDestinationIndex(subID));
		else
			loc2 = AIIndustry.GetLocation(AISubsidy.GetDestinationIndex(subID));
		local station2 = buildTrainStation(loc2, length);
		if (!station2)
			return false; // bugger

		if (!connectTrainStations(station1, length, station2, length)) {
			// check if route complete; build other orientation of depot
			Error("Incomplete route or unable to find suitable depot location");
		} else {
			connectIndustryTrainStations(station1, station2, AISubsidy.GetCargoType(subID), length);
		}
	}

	function connectSubTownsByTrain(subID) {
		biggestTown = AISubsidy.GetSourceIndex(subID);
		
		Debug("First subsidy town: " + AITown.GetName(biggestTown));
		local length1 = abs(sqrt(AITown.GetPopulation(biggestTown)/100));
		local length2 = abs(sqrt(AITown.GetPopulation(biggestTown)/100));
		local station1 = town.BuildTrainStation(biggestTown);
		if(!station1) {
			Error("!!! Building station failed !!!");
			return false;
		}

		local connectTown = AISubsidy.GetDestinationIndex(subID);
		
		Debug("Connecting subsidy town: " + AITown.GetName(connectTown) + " at a manhattan distance of " + AITile.GetDistanceManhattanToTile(AITown.GetLocation(biggestTown), AITown.GetLocation(connectTown)));

		local station2 = town.BuildTrainStation(connectTown);
		if(!station2) {
			Error("!!! Building station failed !!!");
			return false;
		}

		if (connectTrainStations(station1, length1, station2, length2)) {
			local pop1 = AITown.GetPopulation(biggestTown);
			local pop2 = AITown.GetPopulation(connectTown);
			// TODO: see whether this value max sense:
			local divider = AIEngine.GetCapacity(passengerCart) * 10;
			local maxCarts = (pop1 < pop2 ? pop1 : pop2) / divider;	// TODO: change to production in pickup area?
			if (maxCarts < 1)
				maxCarts = 1;
			Debug("Want to deploy " + maxCarts + " carts for this route (pop)");
			connectTownTrainStations(station1, station2, maxCarts);
		}
	}

	/**
	 * 2 stations build, make rail
	 */
	function connectByRail(starts, goals, ignored = []) {
		railFinder.cost.max_cost = 50 * AIMap.DistanceManhattan(starts[0][0], goals[0][0]);
		railFinder.InitializePath(starts, goals, ignored);
		local path = false;
		local i = 0;
		local err;
		local built = 0;
		local iterations = 30000 + 100 * AIMap.DistanceManhattan(starts[0][0], goals[0][0]); // if we pass this; something is wrong
		//local iterations = -1;
		local startTick = GetTick();
		// TODO check whether higher values helps finding faster
		Debug("pathfinding for max " + railFinder.cost.max_cost + " cost and iterations: " + iterations);
		path = railFinder.FindPath(iterations);
		Debug("Total ticks passed: " + (GetTick() - startTick));
		
		if(!path) {
			if (path == false)
				Warning("!! No path found, too little iterations...");
			else
				Warning("!! No path found, too expensive...");
			railFinder.reset();
			return -1;
		}
		Debug("path found for cost: " + path.GetCost() + ", building");

		local prev = null;
		local prevprev = null;
		local prev3 = null;
		local depotBuilt = false;
		local depotLoc = null;
		local railCount = 0;

		while (path != null) {
			if (prevprev != null) {
				if (AIMap.DistanceManhattan(prev, path.GetTile()) > 1) {
					if (AITunnel.GetOtherTunnelEnd(prev) == path.GetTile()) {
						AITunnel.BuildTunnel(AIVehicle.VT_RAIL, prev);
					} else {
						local bridge_list = AIBridgeList_Length(AIMap.DistanceManhattan(path.GetTile(), prev) + 1);
						bridge_list.Valuate(AIBridge.GetMaxSpeed);
						bridge_list.Sort(AIAbstractList.SORT_BY_VALUE, false);
						AIBridge.BuildBridge(AIVehicle.VT_RAIL, bridge_list.Begin(), prev, path.GetTile());
					}
					prevprev = prev;
					prev = path.GetTile();
					path = path.GetParent();
				} else {
					if (!AIRail.BuildRail(prevprev, prev, path.GetTile())) {
						switch(AIError.GetLastError()) {
							case AIError.ERR_VEHICLE_IN_THE_WAY:
								Warning("Building rail failed temporary: vehicle in the way...");
								// try again till the stupid vehicle moved?
								local tries = 100;
								while(!AIRail.BuildRail(prevprev, prev, path.GetTile()) && tries--) {
									Sleep(10);
								}
								break;
							case AIRoad.ERR_ROAD_WORKS_IN_PROGRESS:
								Warning("Building rail failed: road works in progress");
								// good, someone else is building this, skip tile
								break;
							case AIError.ERR_ALREADY_BUILT:
								//Debug("Building rail failed: already built");
								// even better; someone else already built this; silent ignore
								if (AIRail.ConvertRailType(prevprev, prev, current_rail_type))
									Warning("Converted some rail to new type");
								break;
							case AIError.ERR_LAND_SLOPED_WRONG:
								Warning("Building rail failed: wrong land slope");
								// not good; redo from start or terraform?
								return connectByRail(starts, goals, ignores);
							case AIError.ERR_AREA_NOT_CLEAR:
								Warning("Building rail failed: not clear, demolishing tile...");
								AISign.BuildSign(path.GetTile(), ("I'm on your land, demolishing your stuffs"));
								// demolish and retry?
								if (AITile.DemolishTile(path.GetTile())) {
									AIRail.BuildRail(prevprev, prev, path.GetTile());
								} else {
									return connectByRail(starts, goals, ignored);
								}
								break;
							case AIRail.ERR_CROSSING_ON_ONEWAY_ROAD:
								Warning("Can't build crossing on one way road");
								return connectByRail(starts, goals, ignores);
							case AIError.ERR_NOT_ENOUGH_CASH:
								Warning("Panic: Out of cash!!");
								return false;
							case AIError.ERR_OWNED_BY_ANOTHER_COMPANY:
								Warning("Owned by another company; retry...");
								return connectByRail(starts, goals, ignored);
							default:
								Warning("Unknown error during road building " + AIError.GetLastErrorString());
								break;	// no clue what this is; silent ignore for now
						}
					}
				}

				// TODO FIXME
				/*if (railCount++ % 8 == 0) {
					AIRail.BuildSignal(prevprev, prev, AIRail.SIGNALTYPE_PBS_ONEWAY);
				}*/
			}
			if (path != null) {
				if (!depotBuilt && (prevprev != null) && (prev3 != null) && (built++ > 3) && (AIRail.GetRailTracks(prevprev) & AIRail.RAILTRACK_NE_SW) && (AIRail.GetRailTracks(prev) & AIRail.RAILTRACK_NE_SW) && (AIRail.GetRailTracks(prev3) & AIRail.RAILTRACK_NE_SW) ) {
					depotLoc = prevprev + AIMap.GetTileIndex(0,-1);
					depotBuilt = AIRail.BuildRailDepot(depotLoc, prevprev);
					if (depotBuilt) {
						if (!(AIRail.BuildRailTrack(prevprev, AIRail.RAILTRACK_NW_NE) && AIRail.BuildRailTrack(prevprev, AIRail.RAILTRACK_NW_SW))) {
							depotBuilt = false;
							AITile.DemolishTile(depotLoc);
						}
					}
				}
				if (!depotBuilt && (prevprev != null) && (prev3 != null) && (built++ > 3) && (AIRail.GetRailTracks(prevprev) & AIRail.RAILTRACK_NW_SE) && (AIRail.GetRailTracks(prev) & AIRail.RAILTRACK_NW_SE) && (AIRail.GetRailTracks(prev3) & AIRail.RAILTRACK_NW_SE)) {
					depotLoc = prevprev + AIMap.GetTileIndex(-1, 0);
					depotBuilt = AIRail.BuildRailDepot(depotLoc, prevprev);
					if (depotBuilt) {
						if (!(AIRail.BuildRailTrack(prevprev, AIRail.RAILTRACK_NW_NE) && AIRail.BuildRailTrack(prevprev, AIRail.RAILTRACK_NE_SE))) {
							depotBuilt = false;
							AITile.DemolishTile(depotLoc);
						}
					}
				}
				prev3 = prevprev;
				prevprev = prev;
				prev = path.GetTile();
				path = path.GetParent();
			}
		}

		Debug("path done");
		return depotLoc;
	}

	function connectIndustryTrainStations(station1, station2, cargo, cartCount) {
		local depot = myStationsTileList.GetValue(station1);
		local train1 = AIVehicle.BuildVehicle(depot, trainEngine);
		if (!AIVehicle.IsValidVehicle(train1))
			return false;

		local cart;
		local cartType = getCart(cargo, current_rail_type);
		while (cartCount--) {
			cart = AIVehicle.BuildVehicle(depot, cartType);
			if (!AIVehicle.IsValidVehicle(cart)) {
				Error(AIError.GetLastErrorString());
			} else {
				AIVehicle.MoveWagon(cart, 0, train1, 0);
			}
		}

		Debug("Built train: " + train1);
		AIOrder.AppendOrder(train1, station1, AIOrder.AIOF_FULL_LOAD_ANY | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(train1, station2, AIOrder.AIOF_NON_STOP_INTERMEDIATE | AIOrder.AIOF_SERVICE_IF_NEEDED);
		AIVehicle.StartStopVehicle(train1);

		return true;
	}

	function connectTownTrainStations(station1, station2, cartCount) {
		if (cartCount > 12) {
			Warning("TODO: deploy more then 1 train");
			cartCount = 12;
		}

		local depot = myStationsTileList.GetValue(station1);
		local train1 = AIVehicle.BuildVehicle(depot, trainEngine);
		if (!AIVehicle.IsValidVehicle(train1)) {
			Error("Error building train engine");
			return false;
		}
		/*if(!AIVehicle.RefitVehicle(train1, passengerCargoId)) {
			Debug("Error refitting train: " + AIError.GetLastErrorString());
			return false;
		}*/
		local cart;
		local mailCarts = cartCount/4;
		if (cartCount + mailCarts > 12) {
			cartCount = 12 - mailCarts;
		}
		while (mailCarts--) {
			cart = AIVehicle.BuildVehicle(depot, mailCart);
			if (!AIVehicle.IsValidVehicle(cart)) {
				Error(AIError.GetLastErrorString());
			} else {
				AIVehicle.MoveWagon(cart, 0, train1, 0);
			}
		}
		while (cartCount--) {
			cart = AIVehicle.BuildVehicle(depot, passengerCart);
			if (!AIVehicle.IsValidVehicle(cart)) {
				Error(AIError.GetLastErrorString());
			} else {
				AIVehicle.MoveWagon(cart, 0, train1, 0);
			}
		}

		Debug("Built train: " + train1);
		lastPassengerVehicle = train1;
		AIOrder.AppendOrder(train1, station1, AIOrder.AIOF_FULL_LOAD_ANY | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(train1, station2, AIOrder.AIOF_FULL_LOAD_ANY | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(train1, station2, AIOrder.AIOF_GOTO_NEAREST_DEPOT | AIOrder.AIOF_SERVICE_IF_NEEDED);
		AIVehicle.StartStopVehicle(train1);

		/* local train2 = AIVehicle.BuildVehicle(myStationsTileList.GetValue(station2), trainEngine);
		if (!AIVehicle.IsValidVehicle(train2))
			return false; 	// TODO change to return 1 for 1 success
		if(!AIVehicle.RefitVehicle(train2, passengerCargoId)) {
			Debug("Error refitting train: " + AIError.GetLastErrorString());
			return false; // TODO
		}
		Debug("Built train: " + train2);
		AIOrder.AppendOrder(train2, station1, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIOrder.AppendOrder(train2, station2, AIOrder.AIOF_FULL_LOAD | AIOrder.AIOF_NON_STOP_INTERMEDIATE);
		AIVehicle.StartStopVehicle(train2); */

		return true;	// TODO change to 2 
	}
	
	function buildTrainStation(target, length) {
		// TODO: centre isn't always centre; for industry it's the top 
		Debug("Building station...");
		// Find empty block of squares as close to target as possible
		local spotFound = false;
		local curRange = 1;
		local maxRange = 8 + length; //TODO check value correctness (coverage + size of something?)
		if (length > 7)
			length = 7;
		local width = (length == 7 ? 2 : 1);

		// you need changing tracks for multiple platforms
		local totalLength = width > 1 ? length + width : length + 1;
		local area = AITileList();
		Debug("building station (lxb) " + length + " x " + width);

		while (curRange <= maxRange) {
			//Debug("looking with range " + curRange);
			area.AddRectangle(target + AIMap.GetTileIndex(curRange, curRange), target + AIMap.GetTileIndex(-curRange, -curRange));
			area.Valuate(stationLocationValuator, totalLength, width);
			area.RemoveValue(0);
			if (area.Count()) {
				Debug("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					if (AIRail.BuildRailStation(t, area.GetValue(t), width, length, AIStation.STATION_JOIN_ADJACENT)) {

						// TODO: support for wider stations and having exits on the other side
						if (area.GetValue(t) == AIRail.RAILTRACK_NE_SW) {
							local loc = t + AIMap.GetTileIndex(totalLength - 1, 0);
							AIRail.BuildRailTrack(loc, AIRail.RAILTRACK_NE_SW);
							if (width == 1) {
								AIRail.BuildSignal(loc, loc + AIMap.GetTileIndex(-1, 0), AIRail.SIGNALTYPE_PBS);
							} else if (width == 2) {
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 0), AIRail.RAILTRACK_NE_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 1), AIRail.RAILTRACK_NW_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 0), AIRail.RAILTRACK_SW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 0), AIRail.RAILTRACK_NE_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 1), AIRail.RAILTRACK_NE_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 1), AIRail.RAILTRACK_NW_NE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length + 1, 1), AIRail.RAILTRACK_NE_SW);
								AIRail.BuildSignal(loc, loc + AIMap.GetTileIndex(-1, 0), AIRail.SIGNALTYPE_PBS_ONEWAY);
							} else {
								Error("Stations wider then 2 not yet supported");
							}								
						} else if (area.GetValue(t) == AIRail.RAILTRACK_NW_SE) {
							local loc = t + AIMap.GetTileIndex(0, totalLength - 1);
							AIRail.BuildRailTrack(loc, AIRail.RAILTRACK_NW_SE);
							if (width == 1) {
								AIRail.BuildSignal(loc, loc + AIMap.GetTileIndex(0, -1), AIRail.SIGNALTYPE_PBS);
							} else if (width == 2) {
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(0, length), AIRail.RAILTRACK_NW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length), AIRail.RAILTRACK_NW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(0, length), AIRail.RAILTRACK_NW_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(0, length), AIRail.RAILTRACK_SW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length), AIRail.RAILTRACK_NE_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length), AIRail.RAILTRACK_NW_NE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length + 1), AIRail.RAILTRACK_NW_SE);
								AIRail.BuildSignal(loc + AIMap.GetTileIndex(1, 0), loc + AIMap.GetTileIndex(1, -1), AIRail.SIGNALTYPE_PBS_ONEWAY);
							} else {
								Error("Stations wider then 2 not yet supported");
							}								
						}
						return t;
					}
					// TODO: if err == area not clear: demolish?
					//Debug("Error building passenger station " + AIError.GetLastErrorString());
					//AISign.BuildSign(t, AIError.GetLastErrorString());
				}
				curRange++;
			} else {
				curRange++;
			}
		}
		Debug("No possible station location found");
		return null;
	}
	
	function connect2TownsByRail(biggestTown, connectTown) {
		local build1 = false;
		local build2 = false;

		local length1 = abs(sqrt(AITown.GetPopulation(biggestTown)/100));
		local station1 = town.BuildTrainStation(biggestTown);
		if (!station1) {
			Warning("!!! Building station failed in: " + AITown.GetName(biggestTown));
			return false;
		}
		build1 = true;
		myTownList.SetValue(biggestTown, station1);
		
		local length2 = abs(sqrt(AITown.GetPopulation(connectTown)/100));
		local station2 = myTownList.GetValue(connectTown);
		if (station2 == 0) {
			station2 = town.BuildTrainStation(connectTown);
			build2 = true;
		}

		if (!station2) {
			Warning("!!! Building station failed in: " + AITown.GetName(connectTown));
			Warning("Removing other station...");
			// TODO: hand to town and remove signal and rail as well
			if (build1) {
				AITile.DemolishTile(station1);
				myTownList.RemoveValue(station1);
			}
			return false;
		}
		myTownList.SetValue(connectTown, station2);

		if (connectTrainStations(station1, length1, station2, length2)) {
			//we picked BIG towns, lets try to build more carts in 1 go
			local pop1 = AITown.GetPopulation(biggestTown);
			local pop2 = AITown.GetPopulation(connectTown);
			// TODO: see whether this value max sense:
			local divider = AIEngine.GetCapacity(passengerCart) * 10;
			local maxCarts = (pop1 < pop2 ? pop1 : pop2) / divider;	// built a cart for every hundred passengers in the smallest town
			if (maxCarts < 1)
				maxCarts = 1;
			Debug("Want to deploy " + maxCarts + " carts for this route (pop)");
			connectTownTrainStations(station2, station1, maxCarts);
		} else {
			// remove stuff we have build
			if (build1) {
				AITile.DemolishTile(station1);
				myTownList.RemoveValue(station1);
			}
			if (build2) {
				AITile.DemolishTile(station1);
				myTownList.RemoveValue(station1);
			}
		}
	}

	// TODO support for bigger then 7x2
	function connectTrainStations(station1, length1, station2, length2) {
		local extend1 = (length1 >= 7 ? 1 : 0);
		local extend2 = (length2 >= 7 ? 1 : 0);
		if (length1 > 7)
			length1 = 7;
		if (length2 > 7)
			length2 = 7;
		local openingStart;
		local openingGoal;
		local openingStart2;
		local openingGoal2;

		Debug("Route between 2 stations is " + AITile.GetDistanceManhattanToTile(station1, station2));

		if (AIRail.GetRailStationDirection(station1) == AIRail.RAILTRACK_NE_SW) {
			openingStart = station1 + AIMap.GetTileIndex(length1 + extend1 + 1, 0); // TODO FIXME
			openingStart2 = station1 + AIMap.GetTileIndex(length1 + extend1, 0); // TODO FIXME
		} else {
			openingStart = station1 + AIMap.GetTileIndex(0, length1 + extend1 + 1); // TODO FIXME
			openingStart2 = station1 + AIMap.GetTileIndex(0, length1 + extend1); // TODO FIXME
		}

		if (AIRail.GetRailStationDirection(station2) == AIRail.RAILTRACK_NE_SW) {
			openingGoal = station2 + AIMap.GetTileIndex(length2 + extend2 + 1, 0); // TODO FIXME
			openingGoal2 = station2 + AIMap.GetTileIndex(length2 + extend2, 0); // TODO FIXME generalyse
		} else {
			openingGoal = station2 + AIMap.GetTileIndex(0, length2 + extend2 + 1); // TODO FIXME
			openingGoal2 = station2 + AIMap.GetTileIndex(0, length2 + extend2); // TODO FIXME generalyse
		}
		// AISign.BuildSign(openingStart, "F");
		// AISign.BuildSign(openingGoal, "T");
		// AISign.BuildSign(openingStart2, "F");
		// AISign.BuildSign(openingGoal2, "T");
		local starts = [[openingStart, openingStart2]];
		local goals = [[openingGoal, openingGoal2]];
		local stations = [station1, station2];
		local depot = connectByRail(starts, goals, stations);
		if(!AIRail.IsRailDepotTile(depot))
			depot = connectByRail(starts, goals, stations);	// Try once more if it failed


		if(!AIRail.IsRailDepotTile(depot)) {
			// check if route complete; build other orientation of depot
			Error("Incomplete route or unable to find suitable depot location");
			return false;
		} else {
			if (extend1 == 1) {
				if (AIRail.GetRailStationDirection(station1) == AIRail.RAILTRACK_NE_SW) {
					openingStart = station1 + AIMap.GetTileIndex(length1 + extend1 + 1, 1); // TODO FIXME
					openingStart2 = station1 + AIMap.GetTileIndex(length1 + extend1, 1); // TODO FIXME
				} else {
					openingStart = station1 + AIMap.GetTileIndex(1, length1 + extend1 + 1); // TODO FIXME
					openingStart2 = station1 + AIMap.GetTileIndex(1, length1 + extend1); // TODO FIXME
				}
				starts = [[openingStart, openingStart2]];
				connectByRail(starts, goals, stations);
			}
			if (extend2 == 1) {
				if (AIRail.GetRailStationDirection(station2) == AIRail.RAILTRACK_NE_SW) {
					openingGoal = station2 + AIMap.GetTileIndex(length2 + extend2 + 1, 1); // TODO FIXME
					openingGoal2 = station2 + AIMap.GetTileIndex(length2 + extend2, 1); // TODO FIXME generalyse
				} else {
					openingGoal = station2 + AIMap.GetTileIndex(1, length2 + extend2 + 1); // TODO FIXME
					openingGoal2 = station2 + AIMap.GetTileIndex(1, length2 + extend2); // TODO FIXME generalyse
				}
				goals = [[openingGoal, openingGoal2]];
				connectByRail(starts, goals, stations);
			}
			
			myStationsTileList.AddItem(station2, depot);

			// reverse track:
			depot = connectByRail(goals, starts, stations);
			if(AIRail.IsRailDepotTile(depot))
				myStationsTileList.AddItem(station1, depot);

			return true;
		}
	}

	function connectSubsidy(subID) {		
		if (!AISubsidy.IsValidSubsidy(subID)) {
			Error("Invalid subsidy detected");
			return false;
		}
		if (AISubsidy.GetSourceType(subID) == AISubsidy.SPT_TOWN) {
			local town1 = AISubsidy.GetSourceIndex(subID);
			local town2 = AISubsidy.GetDestinationIndex(subID);
			if (AITown.GetPopulation(town1) < 1000 || AITown.GetPopulation(town2) < 1000)
				connectSubTownsByRoad(subID);
			else
				connectSubTownsByTrain(subID);
		} else if (AISubsidy.GetSourceType(subID) == AISubsidy.SPT_INDUSTRY) {
			//connectSubIndustriesByRoad(subID);
			connectSubIndustriesByTrain(subID);
		} else {
			Error("Invalid subsidy detected: " + subID);
		}
	}
}

