/**
 *    This file is part of OtviTrAIn.
 *
 *    OtviTrAIn is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    OtviTrAIn is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with OtviTrAIn.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright 2008-2010 Marnix Bakker, Willem de Neve, Michiel Konstapel and Otto Visser.
 * Suggestions/comments and bugs can go to: rondjeomdekerk@konstapel.nl
 */

require("util.nut");

class OtviTown {
	function constructor() {
		// init town list
	}

	function GetTrainStation(town) {
		// TODO
		return null;
	}

	function trainStationLocationValuator(tile, length, width) { // TODO check in 4 directions!!!
		local landTiles = AITileList();
		if (AITile.IsBuildableRectangle(tile, length + 1, width)) {
			landTiles.AddRectangle(tile, tile + AIMap.GetTileIndex(length - 1, width - 1));
			landTiles.Valuate(AITile.GetSlope);
			landTiles.KeepValue(AITile.SLOPE_FLAT);
			if (landTiles.Count() == length * width) {
				return AIRail.RAILTRACK_NE_SW;
			}
			// Warning("location only has limited flat tiles in NE_SW dir: " + landTiles.Count());
		}
		if (AITile.IsBuildableRectangle(tile, width, length + 1)) {
			landTiles.AddRectangle(tile, tile + AIMap.GetTileIndex(width - 1, length- 1));
			landTiles.Valuate(AITile.GetSlope);
			landTiles.KeepValue(AITile.SLOPE_FLAT);
			if (landTiles.Count() == length * width) {
				return AIRail.RAILTRACK_NW_SE;
			}
			// Warning("location only has limited flat tiles in NW_SE dir: " + landTiles.Count());
		}
		return 0;
	}

	function BuildTrainStation(town) {
		// TODO: centre isn't always centre; for industry it's the top 
		Debug("Building train station...");
		// Find empty block of squares as close to target as possible
		local length = abs(sqrt(AITown.GetPopulation(town)/100));
		local target = AITown.GetLocation(town);
		local spotFound = false;
		local curRange = 1;
		local maxRange = 6 + length; //TODO check value correctness (coverage + size of something?)
		local width = abs(length / 7) + 1;
		length = abs(length % 7) + 1;

		// you need changing tracks for multiple platforms
		local totalLength = width > 1 ? length + width : length + 1;
		local area = AITileList();
		Debug("building station (lxb) " + length + " x " + width);

		while (curRange <= maxRange) {
			//Debug("looking with range " + curRange);
			area.AddRectangle(target + AIMap.GetTileIndex(curRange, curRange), target + AIMap.GetTileIndex(-curRange, -curRange));
			area.Valuate(trainStationLocationValuator, totalLength, width);
			area.RemoveValue(0);
			if (area.Count()) {
				Debug("Found " + area.Count() + " possible options");
				for (local t = area.Begin(); area.HasNext(); t = area.Next()) {
					if (AIRail.BuildRailStation(t, area.GetValue(t), width, length, AIStation.STATION_JOIN_ADJACENT)) {

						// TODO: support for wider stations and having exits on the other side
						if (area.GetValue(t) == AIRail.RAILTRACK_NE_SW) {
							local loc = t + AIMap.GetTileIndex(totalLength - 1, 0);
							AIRail.BuildRailTrack(loc, AIRail.RAILTRACK_NE_SW);
							if (width == 1) {
								AIRail.BuildSignal(loc, loc + AIMap.GetTileIndex(-1, 0), AIRail.SIGNALTYPE_PBS);
							} else if (width == 2) {
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 0), AIRail.RAILTRACK_NE_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 1), AIRail.RAILTRACK_NW_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 0), AIRail.RAILTRACK_SW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 0), AIRail.RAILTRACK_NE_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 1), AIRail.RAILTRACK_NE_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length, 1), AIRail.RAILTRACK_NW_NE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(length + 1, 1), AIRail.RAILTRACK_NE_SW);
								AIRail.BuildSignal(loc, loc + AIMap.GetTileIndex(-1, 0), AIRail.SIGNALTYPE_PBS_ONEWAY);
							} else {
								Error("Stations wider then 2 not yet supported");
							}								
						} else if (area.GetValue(t) == AIRail.RAILTRACK_NW_SE) {
							local loc = t + AIMap.GetTileIndex(0, totalLength - 1);
							AIRail.BuildRailTrack(loc, AIRail.RAILTRACK_NW_SE);
							if (width == 1) {
								AIRail.BuildSignal(loc, loc + AIMap.GetTileIndex(0, -1), AIRail.SIGNALTYPE_PBS);
							} else if (width == 2) {
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(0, length), AIRail.RAILTRACK_NW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length), AIRail.RAILTRACK_NW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(0, length), AIRail.RAILTRACK_NW_SW);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(0, length), AIRail.RAILTRACK_SW_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length), AIRail.RAILTRACK_NE_SE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length), AIRail.RAILTRACK_NW_NE);
								AIRail.BuildRailTrack(t + AIMap.GetTileIndex(1, length + 1), AIRail.RAILTRACK_NW_SE);
								AIRail.BuildSignal(loc + AIMap.GetTileIndex(1, 0), loc + AIMap.GetTileIndex(1, -1), AIRail.SIGNALTYPE_PBS_ONEWAY);
							} else {
								Error("Stations wider then 2 not yet supported");
							}								
						}
						return t;
					}
					// TODO: if err == area not clear: demolish?
					//Debug("Error building passenger station " + AIError.GetLastErrorString());
					//AISign.BuildSign(t, AIError.GetLastErrorString());
				}
				curRange++;
			} else {
				curRange++;
			}
		}
		Debug("No possible station location found");
		return null;
	}

	/**
	 * upgrades small station to bigger one
	 * also reroutes trains to new station if needed
	 */
	function UpgradeTrainStation(town) {}
}

